# Question Hunter

<img style="float:left" src="https://github.com/downloads/marvinpinto/se-question-hunter/screenshot.png"/>

# Contents

 - [Usage](#usage)
 - [About](#about)
 - [Configuration](#configuration)
 - [Contributing](#contributing)
 - [Building](#building)
 - [License](#license)
 - [Download](#download)
 - [Revision History](#revisionhistory)
 - [Platform](#platform)



<a name="usage"></a>
## Usage

    java -jar QuestionHunter-X.X.X/QuestionHunter.jar QuestionHunter-X.X.X/config.json



<a name="about"></a>
## About

**Question Hunter** fetches a targeted list of questions using the Stack
Exchange 2.0 API and assigns each question an MScore. The lowest scoring
questions are then displayed for you to act on.

What is an _MScore_? Well it's a metric I made up. You can refer to it
as "My Score", "Marvin Score", whatever works really.

This program is compatible with all the _main_ Stack Exchange sites, for
example:

 - [Stack Overflow](http://stackoverflow.com/)
 - [Server Fault](http://serverfault.com/)
 - [Super User](http://superuser.com/)
 - [Gaming](http://gaming.stackexchange.com/)
 - [Cooking](http://cooking.stackexchange.com/)
 - [etc.](http://stackexchange.com/sites)


### Motivation

I enjoy answering `android` tagged questions on Stack Overflow but
having to wade through all the _related_ tags that show up with those
questions was quickly getting old. Soon, tag filtering just wasn't
cutting it for me. I was interested in other criteria such as whether a
question had answers (if so, how many, upvoted, downvoted). Were there
any answers by high-rep users, did this question have a log of comments,
and so on.

It turns out the [Stack Exchange 2.0
API](https://api.stackexchange.com/) had a wealth of information
available and writing this program was the next logical step. It allows
me to focus on a set of questions that (very closely) fit the criteria
I'm interested in without having to deal with the _noise_ and
consequently spending more time actually answering questions as opposed
to _searching_ for them.


### Overview

The basic idea behind this program is that each property is assigned a
weight and after the sum total is returned, the question is then either
discarded (scored too high) or displayed (top `n` question). You assign
weights to the properties you value and the lowest (overall) weighted
questions are displayed sorted from lowest to highest.

An important parameter to note is `already_seen_file`; pointing this
parameter at a writable file will ensure that you never see the same
question twice (on successive runs). Both the config file as well as the
`already_seen_file` use the JSON format to store their data.

#### Use cases & examples

1. You like gaining rep (really who doesn't?). You don't feel like
wasting your time on _low hanging fruit_ questions that can be answered
in one line because they're likely to attract a ton of answers and let's
be honest, the probability of getting your answer accepted or upvoted
decreases with every competing answer. Now you can either continue
trolling tag `X` hoping you don't get nailed by the [Fastest Gun in the
West](http://meta.stackoverflow.com/q/9731/176163) problem, or you can
create a filter that will find a few of those _overlooked_ gems and thus
not sacrifice answer quality in the process.

    `overlooked-gems.json`:

    ```json
    {
      "site": "stackoverflow",
      "order": "desc",
      "wanted_tags": [
         "android"
      ],
      "from_date": -1,
      "to_date": 0,
      "q_owner_rep": 0.0001,
      "q_upvote": -1.0,
      "q_downvote": 10,
      "q_owner_accept_rate": 1.0,
      "q_viewcount": 0.005,
      "q_favcount": -0.01,
      "q_comments": 0.5,
      "q_charcount": 0.00,
      "q_age": -0.0001,
      "q_bounty": -0.10,
      "a_owner_rep": 0.01,
      "a_upvote": 5,
      "a_downvote": -1,
      "a_comments": 0.25,
    }
    ```
2. You need to generate a list of old `android` tagged questions that
are likely close-candidates. The following configuration should yield
quite a few that can be acted on:

    `old-android.json`:

    ```json
    {
      "site": "stackoverflow",
      "wanted_tags": [
         "android"
      ],
      "from_date": -730,
      "to_date": -365,
      "order": "asc",
      "q_owner_rep": 0.10,
      "q_upvote": 1.0,
      "q_downvote": -10,
      "q_owner_accept_rate": 0.0,
      "q_viewcount": 0.0,
      "q_favcount": 0.0,
      "q_comments": 1.0,
      "q_charcount": 0.5,
      "q_age": -0.0001,
      "q_bounty": 0.0,
      "a_owner_rep": 0.01,
      "a_upvote": 5,
      "a_downvote": -1,
      "a_comments": 0.25,
      "accepted_answer_restriction": true,
      "user_restriction": false,
      "closed_question_restriction": true,
      "single_tag_restriction": false
    }
    ```

Then to run retrieve these questions:

    java -jar QuestionHunter.jar overlooked-gems.json

Or:

    java -jar QuestionHunter.jar old-android.json



<a name="configuration"></a>
## Configuration

### Configuration Parameters Explained

 - Variable: `display_questions`
    - Type: integer
    - Default: 10
    - The `n` lowest scored questions to display sorted lowest to
      highest.

 - Variable: `max_questions`
    - Type: integer
    - Default: 300
    - The maximum amount of questions to evaluate before choosing the
      best `n` (assuming you set the `display_questions` field value to
      `n`).

 - Variable: `site`
    - Type: string
    - Default: stackoverflow
    - Any valid `api_site_parameter` as returned by
      [this](https://api.stackexchange.com/2.0/sites) query.

 - Variable: `order`
    - Values: asc | desc
    - Default: desc
    - Sort order

 - Variable: `query_pagesize`
    - Type: integer
    - Default: 50
    - Constraints: `{ 0 < query_pagesize <= 100 }`
    - The number of results returned per page - a higher number results
      in less API calls.

 - Variable: `already_seen_file`
    - Type: string
    - Default: null
    - Specify the _full_ path to a file that can be used to store a list
      of _seen_ questions. Using this parameter ensures you won't see
      the same question twice.

 - Variable: `wanted_tags`
    - Type: json string array
    - Default: null
    - List a set of tags you would like to see questions for - this is a
      **required** parameter.

 - Variable: `ignored_tags`
    - Type: json string array
    - Default: null
    - List a set of tags you would *not* like to see, ever. For example,
      suppose you had `android` as a `wanted_tag` and (irritating)
      `jquery` in your `ignored_tags` list, then you would see all
      questions tagged `android` but never `[android, jquery]`. Simply
      put: Let `A` denote the set `wanted_tags` and `B` denote the set
      `ignored_tags`. Then the questions `q` returned will be `{q | q in
      A & q not in B}`. This parameter does substring matching so
      `jquery`, for example would match `jquery-ui`, `mobile-jquery`,
      etc. This is not a required parameter (although I highly recommend
      you use it just so you don't have to deal with jquery!)

 - Variable: `from_date`
    - Type: integer
    - Default: null
    - Constraints: `{ -infinity < from_date < -1 & from_date < to_date }`
    - The lower end of the range of days (back) that you would like to
      see questions for. So `-3` for example would set the lower end of
      the range to three days back. This is a **required** parameter.

 - Variable: `to_date`
    - Type: integer
    - Default: null
    - Constraints: `{ from_date < to_date <= 0 }`
    - Similar to `from_date`, the higher end of the range of days (back)
      that you would like to see questions for. In this context, `0`
      means "today". This is a **required** parameter.

 - Variable: `accepted_answer_restriction`
    - Type: boolean
    - Default: true
    - If enabled, questions with accepted answers will effectively be
      ignored.

 - Variable: `user_restriction`
    - Type: boolean
    - Default: true
    - If enabled, only moderators and registered users' questions are
      considered. Everyone else is ignored.

 - Variable: `closed_question_restriction`
    - Type: boolean
    - Default: true
    - If enabled, questions that have been closed will be ignored.

 - Variable: `single_tag_restriction`
    - Type: boolean
    - Default: true
    - If enabled, questions with *only* one tag (i.e. `android`) will be
      ignored. This is sometimes indicative of a question that hasn't
      been well thought out.

 - Variable: `q_owner_rep`
    - Type: float
    - Default: 0.001
    - This value will be multiplied by the question owner's reputation
      and factored into the weight calculation.

 - Variable: `q_upvote`
    - Type: float
    - Default: -1.0
    - This value will be multiplied by the number of upvotes this
      question has received.

 - Variable: `q_downvote`
    - Type: float
    - Default: 10.0
    - This value will be multiplied by the number of downvotes this
      question has received.

 - Variable: `q_owner_accept_rate`
    - Type: float
    - Default: 2.0
    - This value will be multiplied by the complement of the owner's
      accept rate.

 - Variable: `q_viewcount`
    - Type: float
    - Default: -0.0005
    - This value will be multiplied by the number of views this question
      has had (views as reported by SE).

 - Variable: `q_favcount`
    - Type: float
    - Default: -0.01
    - This value will be multiplied by the number of times this question
      has been "favorited".

 - Variable: `q_comments`
    - Type: float
    - Default: 0.5
    - This value will be multiplied by the number of comments this
      question has received, also factoring in comment
      upvotes. Sometimes indicative of a poorly worded question
      (i.e. constant clarification comments).

 - Variable: `q_charcount`
    - Type: float
    - Default: 0.0
    - This value will be multiplied by the length of the question (in
      characters). Not factored in my default but this can be changed
      via the config file.

 - Variable: `q_age`
    - Type: float
    - Default: -0.0001
    - This value will be multiplied by the age of the question. By
      default, older questions are weighted (slightly) higher than newer
      questions. Use this value with caution - age differences are
      calculated using UNIX epoch time.

 - Variable: `q_bounty`
    - Type: float
    - Default: -0.10
    - This value will be multiplied by the bounty amount on the question
      and factored in to the weight.

 - Variable: `a_owner_rep`
    - Type: float
    - Default: 0.01
    - This value will be multiplied by this answer's owner's reputation
      and factored into the weight.

 - Variable: `a_upvote`
    - Type: float
    - Default: 5.0
    - This value will be multiplied by the number of upvotes this answer
      has received.

 - Variable: `a_downvote`
    - Type: float
    - Default: -1.0
    - This value will be multiplied by the number of downvotes this
      answer has received.

 - Variable: `a_comments`
    - Type: float
    - Default: 0.25
    - This value will be multiplied by the number of comments this
      answer has received, also factoring in comment upvotes.


### Sample Configuration File

`sample-config.json`:

```json
{
  "max_questions": 300,
  "display_questions": 20,
  "site": "stackoverflow",
  "order": "desc",
  "query_pagesize": 95,
  "already_seen_file": "/full/path/to/seen_questions.json",
  "wanted_tags": [
     "android",
     "gson"
  ],
  "ignored_tags": [
     "jquery"
  ],
  "from_date": -5,
  "to_date": 0,
  "q_owner_rep": 0.0001,
  "q_upvote": -1,
  "q_downvote": 10,
  "q_owner_accept_rate": 1,
  "q_viewcount": 0.0005,
  "q_favcount": -0.01,
  "q_comments": 0.5,
  "q_charcount": 0.00,
  "q_age": -0.0001,
  "q_bounty": -0.10,
  "a_owner_rep": 0.01,
  "a_upvote": 5,
  "a_downvote": -1,
  "a_comments": 0.25,
  "accepted_answer_restriction": true,
  "user_restriction": true,
  "closed_question_restriction": true,
  "single_tag_restriction": true
}
```



<a name="contributing"></a>
## Contributing

Please report issues on the [Github issue
tracker](https://github.com/marvinpinto/se-question-hunter/issues). Patches
are preferred as Github pull requests. Please use topic branches when
sending pull requests rather than committing directly to master in order
to minimize unnecessary merge commit clutter.



<a name="building"></a>
## Building

You will need a recent enough version of [Apache
Ant](http://ant.apache.org/) as well as the [Java
SDK](http://www.oracle.com/technetwork/java/javase/downloads/index.html)
in order to build this project.

Then it's a simple matter of:

    $ git clone https://github.com/marvinpinto/se-question-hunter.git
    $ cd se-question-hunter
    $ ant

And assuming everything went well, you will have the packaged `.zip` and
`.tar.gz` files in your `dist` sub-directory. Keep in mind that you will
need to replace the `API_KEY` variable in `ca.marvinp.se.misc.Constants`
with your [Stack Exchange API key](http://stackapps.com/apps/oauth);
otherwise you'll be restricted to [300 API calls per
day](http://stackapps.com/questions/67/how-api-keys-work).



<a name="license"></a>
## License

Copyright 2012 Marvin Pinto (me@marvinp.ca)

Licensed under the Apache License, Version 2.0 (the "License"); you may
not use this file except in compliance with the License. You may obtain
a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This product includes the Apache Commons Lang 3.1 library developed by
the Apache Software Foundation, under the Apache License 2.0.

This product includes the Gson 2.1 library developed by Google Inc.,
under the Apache License 2.0.

This product includes the JSoup 1.6.1 library developed by Jonathan
Hedley, under the MIT license.



<a name="download"></a>
## Download

 - Zip: [QuestionHunter-1.0.1.zip](https://github.com/downloads/marvinpinto/se-question-hunter/QuestionHunter-1.0.1.zip)
 - Gzip: [QuestionHunter-1.0.1.tar.gz](https://github.com/downloads/marvinpinto/se-question-hunter/QuestionHunter-1.0.1.tar.gz)
 - Source code:
   [marvinpinto/se-question-hunter](https://github.com/marvinpinto/se-question-hunter)



<a name="revisionhistory"></a>
## Revision History

 - 1.0.0 (Feb 4, 2012 - Initial version)
 - 1.0.1 (Feb 5, 2012)

   - Updated `build.xml` to output distribution files (zip, tar.gz).
   - Minor updates to the `HELP.txt` file.
   - Minor updates to the `README.md` file.



<a name="platform"></a>
## Platform

Being a Java command-line app, pretty much any system with a functional
[Java Runtime Environment](http://www.java.com/getjava/) should have no issues
running this program.
