/*
  Copyright 2012 Marvin Pinto (me@marvinp.ca)

  Licensed under the Apache License, Version 2.0 (the "License"); you may not
  use this file except in compliance with the License.  You may obtain a copy of
  the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
  License for the specific language governing permissions and limitations under
  the License.

  This product includes the Gson 2.1 library developed by Google Inc., under the
  Apache License 2.0.
*/
package ca.marvinp.se;


import ca.marvinp.se.InterestingQuestions;
import ca.marvinp.se.misc.Settings;
import ca.marvinp.se.misc.Constants;
import ca.marvinp.se.utils.ProgressBar;
import ca.marvinp.se.utils.ApiTracker;
import java.io.File;
import com.google.gson.Gson;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.InputStream;




public class QuestionHunter {


  public static void main(String args[]) {

    if (args.length != 1) {
      System.out.println();
      System.err.println(Constants.ERR_INVALID_ARGS);
      System.err.println(getHelpText());
      System.exit(1);
    }

    // Initialize the API hit counter
    ApiTracker.initialize();

    System.out.print(Constants.MSG_LOADING);
    ProgressBar.update(Constants.PRGB_REG);

    Settings s = getSettings(args[0]);
    if (s == null) {
      System.out.println();
      System.err.println(Constants.ERR_BAD_CONFIG);
      System.exit(1);
    }

    new InterestingQuestions(s);

    // Write out the "seen" questions to disk
    s.writeSeenQuestionstoFile();

    System.exit(0);
  }







  private static String getHelpText() {
    StringBuilder sb = new StringBuilder();
    try {
      InputStream is = QuestionHunter.class.getClassLoader()
        .getResourceAsStream(Constants.README_FILE);
      BufferedReader br = new BufferedReader(new InputStreamReader(is));
      String line;
      while( (line = br.readLine()) != null) {
        sb.append(line + System.getProperty("line.separator"));
      }
      is.close();
    } catch(Exception e) {
      return null;
    }
    return sb.toString();
  }







  private static Settings getSettings(String settingsFile) {
    Settings s = null;
    boolean exists = (new File(settingsFile)).exists();
    if (!exists) {
      return null;
    }

    try {
      ProgressBar.update(Constants.PRGB_REG);
      BufferedReader br = new BufferedReader(new FileReader(settingsFile));
      s = new Gson().fromJson(br, Settings.class);
      ProgressBar.update(Constants.PRGB_REG);
    } catch (Exception e) {
      return null;
    }

    if (!s.areSettingsValid()) {
      return null;
    }

    return s;
  }

}



